'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var fetch = _interopDefault(require('isomorphic-fetch'));

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function serialize(value) {
    let serialized = null;
    try {
        serialized = JSON.stringify(value);
    }
    catch (error) { }
    return serialized;
}
function isPlainObject(value) {
    return typeof value === "object" && value.toString() === "[object Object]";
}
let config = {
    extensionHost: "http://localhost:17341",
};
function configure(newConfig) {
    config = Object.assign(Object.assign({}, config), newConfig);
}
function sendUnserializable(holeId) {
    return fetch(`${config.extensionHost}/unserializable`, {
        method: "POST",
        mode: "cors",
        body: JSON.stringify({
            id: holeId,
        }),
        headers: {
            "Content-Type": "application/json",
        },
    }).catch((err) => console.log(err.message));
}
// This is here so that multiple types would not be edited simultaniously
let sampleQueue = Promise.resolve();
function sendSample(holeId, input) {
    sampleQueue = sampleQueue.then(() => fetch(`${config.extensionHost}/samples`, {
        method: "POST",
        mode: "cors",
        body: JSON.stringify({
            id: holeId,
            sample: input,
        }),
        headers: {
            "Content-Type": "application/json",
        },
    }).catch((err) => console.log(err.message)));
    return sampleQueue;
}
function solveWrapperTypes(value) {
    return __awaiter(this, void 0, void 0, function* () {
        if (typeof (value === null || value === void 0 ? void 0 : value.then) === "function") {
            return {
                __typehole_wrapper_type__: "Promise",
                __typehole_value__: yield value,
            };
        }
        return value;
    });
}
function typeholeFactory(id) {
    const emitSample = sendSample;
    let previousValue = null;
    return function typehole(input) {
        solveWrapperTypes(input).then((withWrapperTypes) => {
            const serialized = serialize(withWrapperTypes);
            if (serialized === previousValue) {
                return input;
            }
            previousValue = serialized;
            if (!serialized ||
                (serialized === "{}" && !isPlainObject(withWrapperTypes))) {
                console.info("Typehole:", "Cannot serialize value", {
                    input,
                    serialized,
                });
                sendUnserializable(id);
            }
            else {
                emitSample(id, withWrapperTypes);
            }
        });
        return input;
    };
}
const holes = {};
var index = new Proxy({}, {
    get: function (target, prop, receiver) {
        if (/^t([0-9]+)?$/.test(prop.toString())) {
            if (!holes[prop]) {
                holes[prop] = typeholeFactory(prop);
            }
            return holes[prop];
        }
        return Reflect.get(target, prop, receiver);
    },
});

exports.configure = configure;
exports.default = index;
